/*= driver mtx3250 ========================================================*/
/* Original Release: 14/02/2012                                            */
/* By: O.L.                                                                */
/* Originally written in C                                                 */
/*=========================================================================*/

#include "rs232.h"
#include "formatio.h"
#include "utility.h"
#include "ansi_c.h"
#include "mtx3250.h"

#define TEMPO	0.1

/*= STATIC VARIABLES ======================================================*/
static int portCom;


/*= UTILITY ROUTINES ======================================================*/
int sendData (char *commande);
int readData (char *buff);
double CoeffMesure (char mult);


/*=========================================================================*/
/* Cette fonction ouvre le port srie , interroge l'appareil connect au   */
/* PC, reset l'appareil.                                                   */
/*=========================================================================*/
int mtx3250_init (int serialPort, int baud, int IDQuery, int resetDevice)
{
	int mtx3250_err = 0;
	char buff[50];

    OpenComConfig (serialPort, "", baud, 0, 8, 1, 512, 512);
    
    if (rs232err == 0)
    	{
    	portCom = serialPort;
	    SetComTime (portCom, 1.0);
	    SetXMode (portCom, 0);
		SetCTSMode (portCom, LWRS_HWHANDSHAKE_OFF);
		
		if (IDQuery)
			{
			mtx3250_err = sendData ("*IDN?\r");
			mtx3250_err = readData (buff);
			if (FindPattern (buff, 0, -1, "METRIX, MTX3250, ", 0, 0) == -1)
				mtx3250_err = -1;
			}

		if (resetDevice)
			mtx3250_err = sendData ("*RST\r");
        }
    else
		mtx3250_err = rs232err;
		
	return mtx3250_err;
}


/*=========================================================================*/
/* Cette fonction configure les mesures de tension de l'appareil	       */
/*=========================================================================*/
int mtx3250_voltage (int function, int range, int secondDisplay)
{
	int mtx3250_err = 0;
	char *Function[3] = {"DC","AC","ACDC"};
	double Range[6] = {0, 0.1, 1, 10, 100, 550};
	char message[200];

	if (function == 0 && (secondDisplay == 1 || secondDisplay == 4 || secondDisplay == 5))
		return -10;
	
	mtx3250_err = sendData ("FUNC \"VOLT\"\r");

	Fmt (message, "INP:COUP %s\r", Function[function]);
	mtx3250_err = sendData (message);
	
	if (range < 0)
		mtx3250_err = sendData ("RANG:AUTO:PEAK 1\r");
	else if (range == 0)
		mtx3250_err = sendData ("RANG:AUTO 1\r");
	else
		{
		Fmt (message, "RANG %f\r", Range[range]);
		mtx3250_err = sendData (message);
		}
		
	Fmt (message, "SENS:SEC %i\r", secondDisplay);
	mtx3250_err = sendData (message);
	
	return mtx3250_err;
}

/*=========================================================================*/
/* Cette fonction configure les mesures de courant de l'appareil	       */
/*=========================================================================*/
int mtx3250_current (int function, int range, int secondDisplay)
{
	int mtx3250_err = 0;
	char *Function[3] = {"DC","AC","ACDC"};
	double Range[6] = {0, 0.0001, 0.001, 0.01, 0.1, 1};
	char message[200];

	if (function == 0 && (secondDisplay == 1 || secondDisplay == 4 || secondDisplay == 5))
		return -10;
	
	mtx3250_err = sendData ("FUNC \"CURR\"\r");

	Fmt (message, "INP:COUP %s\r", Function[function]);
	mtx3250_err = sendData (message);
	
	if (range < 0)
		mtx3250_err = sendData ("RANG:AUTO:PEAK 1\r");
	else if (range == 0)
		mtx3250_err = sendData ("RANG:AUTO 1\r");
	else
		{
		Fmt (message, "RANG %f\r", Range[range]);
		mtx3250_err = sendData (message);
		}
		
	Fmt (message, "SENS:SEC %i\r", secondDisplay);
	mtx3250_err = sendData (message);
	
	return mtx3250_err;
}

/*=========================================================================*/
/* Cette fonction configure les mesures de frquence de l'appareil	       */
/*=========================================================================*/
int mtx3250_frequency (int range, int voltageRange, int secondDisplay)
{
	int mtx3250_err = 0;
	double RangeF[8] = {0, 1, 10, 100, 1000, 10000, 100000, 1000000};
	double RangeV[6] = {0, 0.1, 1, 10, 100, 550};
	char message[200];

	mtx3250_err = sendData ("FUNC \"FREQ\"\r");

	if (range == 0)
		mtx3250_err = sendData ("RANG:AUTO 1\r");
	else
		{
		Fmt (message, "RANG %f\r", RangeF[range]);
		mtx3250_err = sendData (message);
		}
		
	Fmt (message, "FREQ:THR:VOLT:RANG %f\r", RangeV[voltageRange]);
	mtx3250_err = sendData (message);
	
	Fmt (message, "SENS:SEC %i\r", secondDisplay);
	mtx3250_err = sendData (message);
	
	return mtx3250_err;
}

/*=========================================================================*/
/* Cette fonction configure les mesures de resistance et test diode de     */
/* l'appareil														       */
/*=========================================================================*/
int mtx3250_resistance (int second, int range)
{
	int mtx3250_err = 0;
	double Range[7] = {0, 100, 1000, 10000, 100000, 1000000, 10000000};
	char message[200];

	if (second == 1)
		mtx3250_err = sendData ("FUNC \"CONT\"\r");
	else if (second == 2)
		mtx3250_err = sendData ("FUNC \"DIOD\"\r");
	else
		{
		mtx3250_err = sendData ("FUNC \"RES\"\r");

		if (range == 0)
			mtx3250_err = sendData ("RANG:AUTO 1\r");
		else
			{
			Fmt (message, "RANG %f\r", Range[range]);
			mtx3250_err = sendData (message);
			}
		}
		
	return mtx3250_err;
}

/*=========================================================================*/
/* Cette fonction configure les mesures de capacite de l'appareil		   */
/*=========================================================================*/
int mtx3250_capacitance (int range)
{
	int mtx3250_err = 0;
	double Range[8] = {0, 10e-9, 100e-9, 1e-6, 10e-6, 100e-6, 1e-3, 10e-3};
	char message[200];

	mtx3250_err = sendData ("FUNC \"CAPA\"\r");

	if (range == 0)
		mtx3250_err = sendData ("RANG:AUTO 1\r");
	else
		{
		Fmt (message, "RANG %f\r", Range[range]);
		mtx3250_err = sendData (message);
		}
		
	return mtx3250_err;
}

/*=========================================================================*/
/* Cette fonction configure les mesures de capacite de l'appareil		   */
/*=========================================================================*/
int mtx3250_temperature (int unit, int sensor)
{
	int mtx3250_err = 0;
	char *Unit[3] = {"C", "F", "K"};
	char *Sensor[2] = {"PT100", "PT1000"};
	char message[200];

	mtx3250_err = sendData ("FUNC \"TEMP\"\r");

	Fmt (message, "UNIT:TEMP %s\r", Unit[unit]);
	mtx3250_err = sendData (message);
		
	Fmt (message, "TEMP:TRAN %s\r", Sensor[sensor]);
	mtx3250_err = sendData (message);
		
	return mtx3250_err;
}

/*=========================================================================*/
/* Cette fonction configure la rejection de l'appareil		   			   */
/*=========================================================================*/
int mtx3250_rejection (int rejection)
{
	int mtx3250_err = 0;
	char message[200];

	Fmt (message, "MENU:REJEC %i\r", rejection);
	mtx3250_err = sendData (message);

	return mtx3250_err;
}

/*=========================================================================*/
/* Cette fonction configure le filtre de l'appareil			   			   */
/*=========================================================================*/
int mtx3250_filter (int filter)
{
	int mtx3250_err = 0;
	char message[200];

	Fmt (message, "FILT %i\r", filter);
	mtx3250_err = sendData (message);

	return mtx3250_err;
}

/*=========================================================================*/
/* Cette fonction configure l'impedance d'entre de l'appareil			   */
/*=========================================================================*/
int mtx3250_input_impedance (int input_impedance)
{
	int mtx3250_err = 0;
	double Impedance[2] = {1e7, 1e9};
	char message[200];

	Fmt (message, "INP:IMP %f[p1e2]\r", Impedance[input_impedance]);
	mtx3250_err = sendData (message);

	return mtx3250_err;
}

/*=========================================================================*/
/* Cette fonction configure la reference d'impedance pour les calculs dBm  */
/* et Watt de l'appareil			   									   */
/*=========================================================================*/
int mtx3250_ref_ohm (int reference, int value)
{
	int mtx3250_err = 0;
	char message[200];

	if (reference)
		Fmt (message, "MENU:DBM:IMP %i\r", value);
	else
		Fmt (message, "MENU:WATT:IMP %i\r", value);

	mtx3250_err = sendData (message);

	return mtx3250_err;
}

/*=========================================================================*/
/* Cette fonction regle la date et l'heure de l'appareil				   */
/*=========================================================================*/
int mtx3250_date_time (char day, char month, char year, char hour, char minute, char second)
{
	int mtx3250_err = 0;
	char message[200];

	Fmt (message, "SYST:DATE %i,%i,%i\r", year, month, day);
	mtx3250_err = sendData (message);

	Fmt (message, "SYST:TIME %i,%i,%i\r", hour, minute, second);
	mtx3250_err = sendData (message);

	return mtx3250_err;
}

/*=========================================================================*/
/* Cette fonction regle la date et l'heure de l'appareil				   */
/*=========================================================================*/
int mtx3250_beep_contrast (int beep, double contrast)
{
	int mtx3250_err = 0;
	char message[200];

	Fmt (message, "SYST:BEEP:STAT %i\r", beep);
	mtx3250_err = sendData (message);

	Fmt (message, "DISP:CONT %f\r", contrast);
	mtx3250_err = sendData (message);

	return mtx3250_err;
}

/*=========================================================================*/
/* Cette fonction configure le mode print de l'appareil					   */
/*=========================================================================*/
int mtx3250_set_print (int mode, double timer)
{
	int mtx3250_err = 0;
	char message[200];

	if (mode)
		{
		Fmt (message, "HCOPY:RATE %f\r", timer);
		mtx3250_err = sendData (message);
		}

	Fmt (message, "HCOPY:STAT %i\r", mode);
	mtx3250_err = sendData (message);

	return mtx3250_err;
}

/*=========================================================================*/
/* Cette fonction configure le mode math de l'appareil					   */
/*=========================================================================*/
int mtx3250_set_math (int mode, int function, int range, double parameterA, double parameterB, double coef, char unit[])
{
	int mtx3250_err = 0;
	char *Function[4] = {"VOLT", "CURR", "RES", "FREQ"};
	double Range[4][7] = {{0.1, 1, 10, 100, 550, 0},
						  {0.0001, 0.001, 0.01, 0.1, 1, 0},
						  {100, 1000, 10000, 100000, 1000000, 10000000},
						  {1, 10, 100, 1000, 10000, 100000, 1000000}};
	char message[200];

	if ((function < 2 && range > 4) || (function == 2 && range > 5))
		return -10;
	
	if (mode)
		{
		Fmt (message, "CALC:MATH:MFUNC \"%s\"\r", Function[function]);
		mtx3250_err = sendData (message);

		Fmt (message, "CALC:MATH:RANG %f\r", Range[function][range]);
		mtx3250_err = sendData (message);

		Fmt (message, "CALC:MATH:MAF %f\r", parameterA);
		mtx3250_err = sendData (message);

		Fmt (message, "CALC:MATH:MBF %f\r", parameterB);
		mtx3250_err = sendData (message);

		Fmt (message, "CALC:MATH:MCOEFF %f\r", coef);
		mtx3250_err = sendData (message);

		Fmt (message, "CALC:MATH:MUN \"%s\"\r", unit);
		mtx3250_err = sendData (message);
		}

	Fmt (message, "CALC:MATH:STAT %i\r", mode);
	mtx3250_err = sendData (message);

	return mtx3250_err;
}

/*=========================================================================*/
/* Cette fonction configure le mode store de l'appareil					   */
/*=========================================================================*/
int mtx3250_set_store (int mode, double timer, int depth)
{
	int mtx3250_err = 0;
	char message[200];

	if (mode)
		{
		Fmt (message, "TRAC:RATE %f\r", timer);
		mtx3250_err = sendData (message);

		Fmt (message, "TRAC:POIN %i\r", depth);
		mtx3250_err = sendData (message);
		}

	Fmt (message, "TRAC:STO:STAT %i\r", mode);
	mtx3250_err = sendData (message);

	return mtx3250_err;
}

/*=========================================================================*/
/* Cette fonction efface toutes les donnes du mode store de l'appareil	   */
/*=========================================================================*/
int mtx3250_clear_store (void)
{
	return sendData ("TRAC:DEL:ALL\r");
}

/*=========================================================================*/
/* Cette fonction retourne le catalogue du mode store de l'appareil	   	   */
/*=========================================================================*/
int mtx3250_catalog_store (char catalog[])
{
	int mtx3250_err = 0;
	char buffer[500];

	catalog = buffer;
	mtx3250_err = sendData ("TRAC:CAT?\r");
    ComRdTerm (portCom, catalog, 500, 0x0D);
    if (rs232err != 0)
		mtx3250_err = rs232err;
	
	return mtx3250_err;
}

/*=========================================================================*/
/* Cette fonction retourne les donnes d'une campagne de mesure		   	   */
/*=========================================================================*/
int mtx3250_mem_store (int mem, char value[], int NbBytes)
{
	int mtx3250_err = 0;
	char message[200];

	Fmt (message, "TRAC:VAL? MEM%i\r", mem);
	mtx3250_err = sendData (message);

    ComRd (portCom, value, NbBytes - 1);
    if (rs232err != 0)
		mtx3250_err = rs232err;
	
	return mtx3250_err;
}

/*=========================================================================*/
/* Cette fonction lit la mesure en cours							   	   */
/*=========================================================================*/
int mtx3250_measure (double *measure1, char type1[], double *measure2, char type2[], double *measure3, char type3[])
{
	int mtx3250_err = 0;
	char message[200];

	mtx3250_err = sendData ("MEAS?\r");
	mtx3250_err = readData (message);
	
	if (message[1] == ' ')
		message[1] = '0';
	if (message[2] == ' ')
		message[2] = '0';
		
	Scan (message, "%f", measure1);
	(*measure1) *= CoeffMesure (message[7]);
	Fmt (type1, "%s[i8w7]", message);
	
	if (strlen (message) > 20)	// Afficheurs 2 & 3 valides
		{
		if (message[17] == ' ')
			message[17] = '0';
			
		Scan (message + 16, "%f", measure2);
		(*measure2) *= CoeffMesure (message[23]);
		Fmt (type2, "%s[i24w3]", message);

		if (message[30] == ' ')
			message[30] = '0';
			
		Scan (message + 29, "%f", measure3);
		(*measure3) *= CoeffMesure (message[36]);
		Fmt (type3, "%s[i37w3]", message);
		}
	
	return mtx3250_err;
}

/*=========================================================================*/
/* Cette function ferme le port srie                                      */
/*                                                                         */
/*=========================================================================*/
int mtx3250_error (char message[])
{
	int mtx3250_err = 0;

	mtx3250_err = sendData ("SYST:ERR?\r");
	mtx3250_err = readData (message);
	message[FindPattern (message, 0, -1, "\n", 0, 0)] = 0x0;
	
	return mtx3250_err;
}


/*=========================================================================*/
/* Cette function ferme le port srie                                      */
/*                                                                         */
/*=========================================================================*/
int mtx3250_close (void)
{
	int mtx3250_err = 0;

	mtx3250_err = sendData ("SYST:LOC\r");
    mtx3250_err = CloseCom (portCom);
	
	return mtx3250_err;
}


/*=========================================================================*/
/* cette fonction crit dans un buffer de data la chaine de commande       */
/*  envoyer                                                               */
/*=========================================================================*/
int sendData (char *commande)
{
	int mtx3250_err = 0;

	FlushInQ (portCom);
	FlushOutQ (portCom);
    ComWrt (portCom, commande, strlen (commande));
    Delay (TEMPO);
    
    if (rs232err != 0)
		mtx3250_err = rs232err;
		
	return mtx3250_err;
}


/*=========================================================================*/
/* cette fonction lit dans un buffer la chaine de commande renvoye        */
/*                                                                         */
/*=========================================================================*/
int readData (char *buff)
{
	int mtx3250_err = 0;

    ComRdTerm (portCom, buff, 50, 0x0D);

    if (rs232err != 0)
		mtx3250_err = rs232err;
		
	return mtx3250_err;
}

/*=========================================================================*/
/* cette fonction applique le pultiplicateur  la mesure			       */
/*                                                                         */
/*=========================================================================*/
double CoeffMesure (char mult)
	{
	double measure = 1.0;
	
	switch (mult)
		{
		case 'n' : measure = 1e-9; break;
		case '' : measure = 1e-6; break;
		case 'm' : measure = 1e-3; break;
		case 'k' : measure = 1e+3; break;
		case 'M' : measure = 1e+6; break;
		case 'G' : measure = 1e+9; break;
		}
		
	return measure;
	}
